@@ -43,6 +43,7 @@ int main(int argc, char* argv[])
   QCoreApplication::setOrganizationName("OpenChemistry");
   QCoreApplication::setOrganizationDomain("openchemistry.org");
   QCoreApplication::setApplicationName("Avogadro");
+  QGuiApplication::setDesktopFileName("avogadro2");
 
   QCoreApplication::setAttribute(Qt::AA_UseHighDpiPixmaps);
   QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
@@ -90,10 +91,10 @@ int main(int argc, char* argv[])
 
   // Make sure to use pointers:
   //
-  QTranslator* qtTranslator = new QTranslator;
-  QTranslator* qtBaseTranslator = new QTranslator;
-  QTranslator* avoTranslator = new QTranslator;
-  QTranslator* avoLibsTranslator = new QTranslator;
+  auto* qtTranslator = new QTranslator;
+  auto* qtBaseTranslator = new QTranslator;
+  auto* avoTranslator = new QTranslator;
+  auto* avoLibsTranslator = new QTranslator;
   bool qtLoaded = false;
   bool avoLoaded = false;
   bool libsLoaded = false;
@@ -151,10 +152,22 @@ int main(int argc, char* argv[])
 
   languages << "System"; // we handle this in the dialog
   codes << ""; // default is the system language
+
+  bool addedUS = false;
+
   // check what files exist
   foreach (const QString& file, files) {
     // remove "avogadroapp-" and the ".qm"
     QString localeCode = file.left(file.indexOf('.')).remove("avogadroapp-");
+
+    if (localeCode.startsWith("en") && !addedUS) {
+      // add US English (default)
+      addedUS = true;
+      QLocale us("en_US");
+      languages << us.nativeLanguageName();
+      codes << "en_US";
+    }
+
     QLocale locale(localeCode);
     QString languageName = locale.nativeLanguageName();
     if (languageName.isEmpty() && localeCode == "oc")
@@ -182,7 +195,7 @@ int main(int argc, char* argv[])
 
   if (!contextIsValid) {
     QMessageBox::information(
-      0, QCoreApplication::translate("main.cpp", "Avogadro"),
+      nullptr, QCoreApplication::translate("main.cpp", "Avogadro"),
       QCoreApplication::translate("main.cpp",
                                   "This system does not support OpenGL."));
     return 1;
@@ -229,13 +242,12 @@ int main(int argc, char* argv[])
     }
   }
 
-  Avogadro::MainWindow* window =
-    new Avogadro::MainWindow(fileNames, disableSettings);
-  window->setTranslationList(languages, codes);
+  Avogadro::MainWindow window(fileNames, disableSettings);
+  window.setTranslationList(languages, codes);
 #ifdef QTTESTING
-  window->playTest(testFile, testExit);
+  window.playTest(testFile, testExit);
 #endif
-  window->show();
+  window.show();
 
 #ifdef Avogadro_ENABLE_RPC
   // create rpc listener
