@@ -9,6 +9,8 @@
 #include "avogadroappconfig.h"
 #include "backgroundfileformat.h"
 #include "menubuilder.h"
+#include "renderingdialog.h"
+#include "tdxcontroller.h"
 #include "tooltipfilter.h"
 #include "viewfactory.h"
 
@@ -38,36 +40,44 @@
 #include <QtCore/QDebug>
 #include <QtCore/QFile>
 #include <QtCore/QFileInfo>
+#include <QtCore/QJsonDocument>
+#include <QtCore/QJsonObject>
 #include <QtCore/QMimeData>
+#include <QtCore/QProcess>
 #include <QtCore/QSettings>
 #include <QtCore/QStandardPaths>
 #include <QtCore/QString>
 #include <QtCore/QThread>
 #include <QtCore/QTimer>
+
+#include <QOpenGLFramebufferObject>
 #include <QtGui/QClipboard>
 #include <QtGui/QCloseEvent>
 #include <QtGui/QDesktopServices>
 #include <QtGui/QKeySequence>
-#include <QtWidgets/QActionGroup>
+
+#include <QtNetwork/QNetworkAccessManager>
+#include <QtNetwork/QNetworkReply>
+
+#include <QActionGroup>
 #include <QtWidgets/QApplication>
+#include <QtWidgets/QColorDialog>
+#include <QtWidgets/QDockWidget>
 #include <QtWidgets/QFileDialog>
 #include <QtWidgets/QHBoxLayout>
+#include <QtWidgets/QHeaderView>
 #include <QtWidgets/QInputDialog>
 #include <QtWidgets/QMenu>
 #include <QtWidgets/QMenuBar>
 #include <QtWidgets/QMessageBox>
-#include <QtWidgets/QStatusBar>
-#include <QtWidgets/QToolBar>
-
-#include <QtWidgets/QColorDialog>
-#include <QtWidgets/QDockWidget>
-#include <QtWidgets/QHeaderView>
 #include <QtWidgets/QProgressDialog>
 #include <QtWidgets/QPushButton>
+#include <QtWidgets/QStatusBar>
+#include <QtWidgets/QToolBar>
+#include <QtWidgets/QToolButton>
 #include <QtWidgets/QTreeView>
 
-#include <QToolButton>
-#include <QtGui/QOpenGLFramebufferObject>
+#include <QScreen>
 
 #ifdef QTTESTING
 #include <QXmlStreamReader>
@@ -236,6 +246,9 @@ MainWindow::MainWindow(const QStringList& fileNames, bool disableSettings)
   , m_redo(nullptr)
   , m_copyImage(nullptr)
   , m_viewFactory(new ViewFactory)
+#ifdef _3DCONNEXION
+  , m_TDxController(nullptr)
+#endif
 {
   // If disable settings, ensure we create a cleared QSettings object.
   if (disableSettings) {
@@ -246,6 +259,9 @@ MainWindow::MainWindow(const QStringList& fileNames, bool disableSettings)
   // The default settings will be used if everything was cleared.
   readSettings();
 
+  // check for version update
+  checkUpdate();
+
   // Now load the plugins.
   PluginManager* plugin = PluginManager::instance();
   plugin->load();
@@ -257,7 +273,8 @@ MainWindow::MainWindow(const QStringList& fileNames, bool disableSettings)
     plugin->pluginFactories<ExtensionPluginFactory>();
   qDebug() << "Extension plugins dynamically found…" << extensions.size();
   foreach (ExtensionPluginFactory* factory, extensions) {
-    ExtensionPlugin* extension = factory->createInstance();
+    ExtensionPlugin* extension =
+      factory->createInstance(QCoreApplication::instance());
     if (extension) {
       extension->setParent(this);
       connect(this, &MainWindow::moleculeChanged, extension,
@@ -270,6 +287,10 @@ MainWindow::MainWindow(const QStringList& fileNames, bool disableSettings)
               &MainWindow::setActiveTool);
       connect(extension, &QtGui::ExtensionPlugin::requestActiveDisplayTypes,
               this, &MainWindow::setActiveDisplayTypes);
+      connect(extension, &QtGui::ExtensionPlugin::registerCommand, this,
+              &MainWindow::registerExtensionCommand);
+      extension->registerCommands();
+
       buildMenu(extension);
       m_extensions.append(extension);
     }
@@ -299,10 +320,35 @@ MainWindow::MainWindow(const QStringList& fileNames, bool disableSettings)
   statusBar()->showMessage(tr("Ready…"), 2000);
 
   updateWindowTitle();
+
+#ifdef _3DCONNEXION
+  GLWidget* glWidget =
+    qobject_cast<GLWidget*>(m_multiViewWidget->activeWidget());
+
+  m_TDxController = new TDxController(this, glWidget);
+
+  connect(this, &MainWindow::moleculeChanged, m_TDxController,
+          &TDxController::updateMolecule);
+
+  m_TDxController->enableController();
+
+  QMap<QString, QList<QAction*>> actionsMap = m_menuBuilder->getMenuActions();
+  QList<QAction*> toolActions;
+
+  for (auto tool : m_tools)
+    toolActions.push_back(tool->activateAction());
+
+  actionsMap.insert("Toolbox", toolActions);
+
+  m_TDxController->exportCommands(actionsMap);
+#endif
 }
 
 MainWindow::~MainWindow()
 {
+#ifdef _3DCONNEXION
+  m_TDxController->disableController();
+#endif
   writeSettings();
   delete m_molecule;
   delete m_menuBuilder;
@@ -319,10 +365,10 @@ void MainWindow::setupInterface()
   m_multiViewWidget = new QtGui::MultiViewWidget(this);
   m_multiViewWidget->setFactory(m_viewFactory);
   setCentralWidget(m_multiViewWidget);
-  GLWidget* glWidget = new GLWidget(this);
+  auto* glWidget = new GLWidget(this);
 
   // set the background color (alpha channel default should be opaque)
-  QColor color =
+  auto color =
     settings.value("backgroundColor", QColor(0, 0, 0, 255)).value<QColor>();
   Vector4ub cColor;
   cColor[0] = static_cast<unsigned char>(color.red());
@@ -336,46 +382,57 @@ void MainWindow::setupInterface()
   m_multiViewWidget->addWidget(glWidget);
   ActiveObjects::instance().setActiveGLWidget(glWidget);
 
+  // set solid pipeline parameters
+  Rendering::SolidPipeline* pipeline = &glWidget->renderer().solidPipeline();
+  if (pipeline) {
+    pipeline->setAoEnabled(
+      settings.value("MainWindow/ao_enabled", true).toBool());
+    pipeline->setAoStrength(
+      settings.value("MainWindow/ao_strength", 1.0f).toFloat());
+    pipeline->setEdEnabled(
+      settings.value("MainWindow/ed_enabled", true).toBool());
+  }
+
   // Our tool dock.
   m_toolDock = new QDockWidget(tr("Tool"), this);
   addDockWidget(Qt::LeftDockWidgetArea, m_toolDock);
 
   // Our scene/view dock.
   m_sceneDock = new QDockWidget(tr("Display Types"), this);
-  m_sceneDock->setFeatures(m_sceneDock->features() &
-                           ~QDockWidget::AllDockWidgetFeatures);
-
   m_sceneTreeView = new QTreeView(m_sceneDock);
   m_sceneTreeView->setIndentation(0);
   m_sceneTreeView->setSelectionBehavior(QAbstractItemView::SelectRows);
   m_sceneTreeView->setSelectionMode(QAbstractItemView::SingleSelection);
   m_sceneDock->setWidget(m_sceneTreeView);
   addDockWidget(Qt::LeftDockWidgetArea, m_sceneDock);
+
   // Our view dock.
   m_viewDock = new QDockWidget(tr("View Configuration"), this);
   addDockWidget(Qt::LeftDockWidgetArea, m_viewDock);
-  m_viewDock->setFeatures(m_viewDock->features() &
-                          ~QDockWidget::AllDockWidgetFeatures);
 
   // Our molecule dock.
-  QDockWidget* moleculeDock = new QDockWidget(tr("Molecules"), this);
-  m_moleculeTreeView = new QTreeView(moleculeDock);
+  m_moleculeDock = new QDockWidget(tr("Molecules"), this);
+  m_moleculeTreeView = new QTreeView(m_moleculeDock);
   m_moleculeTreeView->setIndentation(0);
   m_moleculeTreeView->setSelectionBehavior(QAbstractItemView::SelectRows);
   m_moleculeTreeView->setSelectionMode(QAbstractItemView::SingleSelection);
-  moleculeDock->setWidget(m_moleculeTreeView);
-  addDockWidget(Qt::LeftDockWidgetArea, moleculeDock);
+  m_moleculeDock->setWidget(m_moleculeTreeView);
+  addDockWidget(Qt::LeftDockWidgetArea, m_moleculeDock);
 
   // Our molecule dock.
-  QDockWidget* layerDock = new QDockWidget(tr("Layers"), this);
-  m_layerTreeView = new QTreeView(layerDock);
+  m_layerDock = new QDockWidget(tr("Layers"), this);
+  m_layerTreeView = new QTreeView(m_layerDock);
   m_layerTreeView->setIndentation(0);
   m_layerTreeView->setSelectionBehavior(QAbstractItemView::SelectRows);
   m_layerTreeView->setSelectionMode(QAbstractItemView::SingleSelection);
-  layerDock->setWidget(m_layerTreeView);
-  addDockWidget(Qt::LeftDockWidgetArea, layerDock);
+  m_layerDock->setWidget(m_layerTreeView);
+  addDockWidget(Qt::LeftDockWidgetArea, m_layerDock);
 
-  tabifyDockWidget(moleculeDock, layerDock);
+  // this doesn't seem necessary
+  //  m_layerDock->setVisible(settings.value("layerDock", true).toBool());
+
+  tabifyDockWidget(m_moleculeDock, m_layerDock);
+  m_moleculeDock->raise();
 
   // Switch to our fallback icons if there are no platform-specific icons.
   if (!QIcon::hasThemeIcon("document-new"))
@@ -485,9 +542,9 @@ void MainWindow::dropEvent(QDropEvent* event)
 
 void MainWindow::moleculeReady(int)
 {
-  ExtensionPlugin* extension = qobject_cast<ExtensionPlugin*>(sender());
+  auto* extension = qobject_cast<ExtensionPlugin*>(sender());
   if (extension) {
-    Molecule* mol = new Molecule(this);
+    auto* mol = new Molecule(this);
     if (extension->readMolecule(*mol))
       setMolecule(mol);
   }
@@ -510,8 +567,7 @@ void setDefaultViews(MultiViewWidget* viewWidget)
 {
   QSettings settings;
   // save the enabled scene / render plugins
-  if (GLWidget* glWidget =
-        qobject_cast<GLWidget*>(viewWidget->activeWidget())) {
+  if (auto* glWidget = qobject_cast<GLWidget*>(viewWidget->activeWidget())) {
 
     const ScenePluginModel* sceneModel = &glWidget->sceneModel();
     bool anyPluginTrue = false;
@@ -583,11 +639,11 @@ void MainWindow::setMolecule(Molecule* mol)
 
   // Check if the molecule needs to update the current one.
   QWidget* w = m_multiViewWidget->activeWidget();
-  if (GLWidget* glWidget = qobject_cast<QtOpenGL::GLWidget*>(w)) {
+  if (auto* glWidget = qobject_cast<QtOpenGL::GLWidget*>(w)) {
     setWidgetMolecule(glWidget, mol);
   }
 #ifdef AVO_USE_VTK
-  else if (vtkGLWidget* vtkWidget = qobject_cast<vtkGLWidget*>(w)) {
+  else if (auto* vtkWidget = qobject_cast<vtkGLWidget*>(w)) {
     setWidgetMolecule(vtkWidget, mol);
   }
 #endif
@@ -657,8 +713,15 @@ void MainWindow::writeSettings()
   settings.endGroup();
   settings.setValue("recentFiles", m_recentFiles);
 
+  // save which docks are visible
+  settings.setValue("viewDock", m_viewDock->isVisible());
+  settings.setValue("toolDock", m_toolDock->isVisible());
+  settings.setValue("sceneDock", m_sceneDock->isVisible());
+  settings.setValue("layerDock", m_layerDock->isVisible());
+  settings.setValue("moleculeDock", m_moleculeDock->isVisible());
+
   // save the enabled scene / render plugins
-  if (GLWidget* glWidget =
+  if (auto* glWidget =
         qobject_cast<GLWidget*>(m_multiViewWidget->activeWidget())) {
 
     const ScenePluginModel* sceneModel = &glWidget->sceneModel();
@@ -763,7 +826,8 @@ bool MainWindow::addScript(const QString& filePath)
   // TODO: add some sort of warning?
   QStringList types;
   types << tr("Commands") << tr("Input Generators") << tr("File Formats")
-        << tr("Charges", "atomic electrostatics");
+        << tr("Charges", "atomic electrostatics")
+        << tr("Force Fields", "potential energy calculators");
 
   bool ok;
   QString item =
@@ -790,6 +854,9 @@ bool MainWindow::addScript(const QString& filePath)
     case 4:
       typePath = "charges";
       break;
+    case 5:
+      typePath = "energy";
+      break;
     default:
       typePath = "other";
   }
@@ -938,8 +1005,8 @@ bool MainWindow::backgroundWriterFinished()
 
 void MainWindow::toolActivated()
 {
-  if (QAction* action = qobject_cast<QAction*>(sender())) {
-    if (GLWidget* glWidget =
+  if (auto* action = qobject_cast<QAction*>(sender())) {
+    if (auto* glWidget =
           qobject_cast<GLWidget*>(m_multiViewWidget->activeWidget())) {
       glWidget->setActiveTool(action->data().toString());
       if (glWidget->activeTool()) {
@@ -960,7 +1027,7 @@ void MainWindow::viewConfigActivated() {}
 
 void MainWindow::rendererInvalid()
 {
-  GLWidget* widget = qobject_cast<GLWidget*>(sender());
+  auto* widget = qobject_cast<GLWidget*>(sender());
   MESSAGEBOX::warning(this, tr("Error: Failed to initialize OpenGL context"),
                       tr("OpenGL 2.0 or greater required, exiting.\n\n%1")
                         .arg(widget ? widget->error() : tr("Unknown error")));
@@ -1003,11 +1070,11 @@ void MainWindow::layerActivated(const QModelIndex& idx)
 
     if (updateGL) {
       QWidget* w = m_multiViewWidget->activeWidget();
-      if (GLWidget* glWidget = qobject_cast<QtOpenGL::GLWidget*>(w)) {
+      if (auto* glWidget = qobject_cast<QtOpenGL::GLWidget*>(w)) {
         glWidget->updateScene();
       }
 #ifdef AVO_USE_VTK
-      else if (vtkGLWidget* vtkWidget = qobject_cast<vtkGLWidget*>(w)) {
+      else if (auto* vtkWidget = qobject_cast<vtkGLWidget*>(w)) {
         vtkWidget->updateScene();
       }
 #endif
@@ -1018,8 +1085,8 @@ void MainWindow::layerActivated(const QModelIndex& idx)
 
 void MainWindow::moleculeActivated(const QModelIndex& idx)
 {
-  QObject* obj = static_cast<QObject*>(idx.internalPointer());
-  if (Molecule* mol = qobject_cast<Molecule*>(obj)) {
+  auto* obj = static_cast<QObject*>(idx.internalPointer());
+  if (auto* mol = qobject_cast<Molecule*>(obj)) {
     if (idx.column() == 0)
       setMolecule(mol);
 
@@ -1045,14 +1112,15 @@ void MainWindow::sceneItemActivated(const QModelIndex& idx)
 {
   if (!idx.isValid())
     return;
-  QObject* obj = static_cast<QObject*>(idx.internalPointer());
-  if (ScenePlugin* scene = qobject_cast<ScenePlugin*>(obj)) {
+  auto* obj = static_cast<QObject*>(idx.internalPointer());
+  if (auto* scene = qobject_cast<ScenePlugin*>(obj)) {
     m_viewDock->setWidget(scene->setupWidget());
     m_activeScenePlugin = scene;
   }
 }
 
-bool populatePluginModel(ScenePluginModel& model, bool editOnly = false)
+bool populatePluginModel(ScenePluginModel& model, QObject* p,
+                         bool editOnly = false)
 {
   if (!model.scenePlugins().empty())
     return false;
@@ -1062,7 +1130,7 @@ bool populatePluginModel(ScenePluginModel& model, bool editOnly = false)
   QList<ScenePluginFactory*> scenePluginFactories =
     plugin->pluginFactories<ScenePluginFactory>();
   foreach (ScenePluginFactory* factory, scenePluginFactories) {
-    ScenePlugin* scenePlugin = factory->createInstance();
+    ScenePlugin* scenePlugin = factory->createInstance(p);
     if (editOnly && scenePlugin) {
       if (scenePlugin->objectName() == "BallStick") {
         model.addItem(scenePlugin);
@@ -1085,20 +1153,20 @@ bool populateTools(T* glWidget)
   QList<ToolPluginFactory*> toolPluginFactories =
     plugin->pluginFactories<ToolPluginFactory>();
   foreach (ToolPluginFactory* factory, toolPluginFactories) {
-    ToolPlugin* tool = factory->createInstance();
+    ToolPlugin* tool = factory->createInstance(QCoreApplication::instance());
     if (tool)
       glWidget->addTool(tool);
   }
-  glWidget->setDefaultTool(QObject::tr("Navigate tool"));
-  glWidget->setActiveTool(QObject::tr("Navigate tool"));
+  glWidget->setDefaultTool("Navigator");
+  glWidget->setActiveTool("Navigator");
   return true;
 }
 
 void MainWindow::viewActivated(QWidget* widget)
 {
   ActiveObjects::instance().setActiveWidget(widget);
-  if (GLWidget* glWidget = qobject_cast<GLWidget*>(widget)) {
-    bool firstRun = populatePluginModel(glWidget->sceneModel());
+  if (auto* glWidget = qobject_cast<GLWidget*>(widget)) {
+    bool firstRun = populatePluginModel(glWidget->sceneModel(), this);
     m_sceneTreeView->setModel(&glWidget->sceneModel());
     populateTools(glWidget);
 
@@ -1124,7 +1192,7 @@ void MainWindow::viewActivated(QWidget* widget)
       }
     }
     if (m_molecule != glWidget->molecule() && glWidget->molecule()) {
-      m_rwMolecule = 0;
+      m_rwMolecule = nullptr;
       m_molecule = glWidget->molecule();
       emit moleculeChanged(m_molecule);
       m_moleculeModel->setActiveMolecule(m_molecule);
@@ -1133,8 +1201,8 @@ void MainWindow::viewActivated(QWidget* widget)
     ActiveObjects::instance().setActiveGLWidget(glWidget);
   }
 #ifdef AVO_USE_VTK
-  else if (vtkGLWidget* vtkWidget = qobject_cast<vtkGLWidget*>(widget)) {
-    bool firstRun = populatePluginModel(vtkWidget->sceneModel());
+  else if (auto* vtkWidget = qobject_cast<vtkGLWidget*>(widget)) {
+    bool firstRun = populatePluginModel(vtkWidget->sceneModel(), this);
     m_sceneTreeView->setModel(&vtkWidget->sceneModel());
 
     if (firstRun) {
@@ -1161,7 +1229,7 @@ QImage MainWindow::renderToImage(const QSize& size)
 {
   QImage exportImage(size, QImage::Format_ARGB32);
 
-  QOpenGLWidget* glWidget =
+  auto* glWidget =
     qobject_cast<QOpenGLWidget*>(m_multiViewWidget->activeWidget());
 
   // render it (with alpha channel)
@@ -1181,7 +1249,8 @@ QImage MainWindow::renderToImage(const QSize& size)
   if (QOpenGLFramebufferObject::hasOpenGLFramebufferObjects()) {
     exportImage = glWidget->grabFramebuffer();
   } else {
-    QPixmap pixmap = QPixmap::grabWindow(glWidget->winId());
+    auto* screen = QGuiApplication::primaryScreen();
+    auto pixmap = screen->grabWindow(glWidget->winId());
     exportImage = pixmap.toImage();
   }
 
@@ -1205,6 +1274,7 @@ QImage MainWindow::renderToImage(const QSize& size)
 
 void MainWindow::exportGraphics()
 {
+  // ask the user for a filename
   QStringList filters;
 // Omit "common image formats" on Mac
 #ifdef Q_OS_MAC
@@ -1224,6 +1294,11 @@ void MainWindow::exportGraphics()
   QString fileName = QFileDialog::getSaveFileName(
     this, tr("Export Bitmap Graphics"), "", "PNG (*.png)");
 
+  exportGraphics(fileName);
+}
+
+void MainWindow::exportGraphics(QString fileName)
+{
   if (fileName.isEmpty())
     return;
   if (QFileInfo(fileName).suffix().isEmpty())
@@ -1255,7 +1330,7 @@ void MainWindow::openRecentFile()
   if (!saveFileIfNeeded())
     return;
 
-  QAction* action = qobject_cast<QAction*>(sender());
+  auto* action = qobject_cast<QAction*>(sender());
   if (action) {
     QString fileName = action->data().toString();
 
@@ -1296,9 +1371,9 @@ bool MainWindow::saveFile(bool async)
   if (!molObj)
     return false;
 
-  Molecule* mol = qobject_cast<Molecule*>(molObj);
+  auto* mol = qobject_cast<Molecule*>(molObj);
   if (!mol) {
-    RWMolecule* rwMol = qobject_cast<RWMolecule*>(molObj);
+    auto* rwMol = qobject_cast<RWMolecule*>(molObj);
     if (!rwMol)
       return false;
     return saveFileAs(async);
@@ -1367,8 +1442,10 @@ bool MainWindow::saveFileAs(bool async)
   QSettings settings;
   QString dir = settings.value("MainWindow/lastSaveDir").toString();
 
-  QString fileName =
-    QFileDialog::getSaveFileName(this, tr("Save chemical file"), dir, filter);
+  QFileDialog saveDialog(this, tr("Save chemical file"), dir, filter);
+  saveDialog.setAcceptMode(QFileDialog::AcceptSave);
+  saveDialog.exec();
+  QString fileName = saveDialog.selectedFiles().first();
 
   if (fileName.isEmpty()) // user cancel
     return false;
@@ -1377,15 +1454,25 @@ bool MainWindow::saveFileAs(bool async)
   dir = info.absoluteDir().absolutePath();
   settings.setValue("MainWindow/lastSaveDir", dir);
 
-  // Create one of our writers to save the file:
+  // Use manually entered extension if present
   QString extension = info.suffix().toLower();
+  // Otherwise, get extension from selected filter
+  if (extension.isEmpty()) {
+    QString filter = saveDialog.selectedNameFilter();
+    if (filter.contains("(*.cml)", Qt::CaseSensitive))
+      extension = "cml";
+    else
+      extension = "cjson";
+
+    fileName += "." + extension;
+  }
+
+  // Create one of our writers to save the file:
   FileFormat* writer = nullptr;
   if (extension == "cjson" || extension.isEmpty())
     writer = new Io::CjsonFormat;
   else if (extension == "cml")
     writer = new Io::CmlFormat;
-  if (extension.isEmpty())
-    fileName += ".cjson";
 
   return saveFileAs(fileName, writer, async);
 }
@@ -1407,6 +1494,38 @@ bool MainWindow::exportFile(bool async)
   return saveFileAs(reply.second, reply.first->newInstance(), async);
 }
 
+bool MainWindow::exportFile(const QString& fileName, bool async)
+{
+  if (fileName.isEmpty()) {
+    return false;
+  }
+
+  // Create one of our writers to save the file:
+  FileFormat* writer = nullptr;
+
+  std::vector<const FileFormat*> writers =
+    Io::FileFormatManager::instance().fileFormatsFromFileExtension(
+      QFileInfo(fileName).suffix().toStdString(),
+      FileFormat::File | FileFormat::Write);
+
+  if (!writers.empty()) {
+    writer = writers[0]->newInstance();
+    return saveFileAs(fileName, writer, async);
+  }
+
+  return false;
+}
+
+std::string MainWindow::exportString(const std::string& format)
+{
+  std::string output;
+  auto* mol = qobject_cast<Molecule*>(m_moleculeModel->activeMolecule());
+
+  Io::FileFormatManager::instance().writeString(*mol, output, format);
+
+  return output;
+}
+
 bool MainWindow::saveFileAs(const QString& fileName, Io::FileFormat* writer,
                             bool async)
 {
@@ -1432,7 +1551,7 @@ bool MainWindow::saveFileAs(const QString& fileName, Io::FileFormat* writer,
     m_threadedWriter->deleteLater();
   m_threadedWriter = new BackgroundFileFormat(writer);
 
-  Molecule* mol = qobject_cast<Molecule*>(molObj);
+  auto* mol = qobject_cast<Molecule*>(molObj);
   if (!mol) {
     delete writer;
     return false;
@@ -1478,7 +1597,7 @@ bool MainWindow::saveFileAs(const QString& fileName, Io::FileFormat* writer,
 
 void MainWindow::setActiveTool(QString toolName)
 {
-  if (GLWidget* glWidget =
+  if (auto* glWidget =
         qobject_cast<GLWidget*>(m_multiViewWidget->activeWidget())) {
     foreach (ToolPlugin* toolPlugin, glWidget->tools()) {
       if (toolPlugin->objectName() == toolName) {
@@ -1520,12 +1639,43 @@ void MainWindow::setActiveDisplayTypes(QStringList displayTypes)
   }
 #endif
 
-  foreach (ScenePlugin* scene, scenePluginModel->scenePlugins())
-    scene->setEnabled(false);
+  //  foreach (ScenePlugin* scene, scenePluginModel->scenePlugins())
+  //    scene->setEnabled(false);
   foreach (ScenePlugin* scene, scenePluginModel->scenePlugins())
     foreach (const QString& name, displayTypes)
       if (scene->objectName() == name)
         scene->setEnabled(true);
+
+  if (glWidget)
+    glWidget->updateScene();
+#ifdef AVO_USE_VTK
+  else if (vtkWidget)
+    vtkWidget->updateScene();
+#endif
+}
+
+void MainWindow::setDisabledDisplayTypes(QStringList displayTypes)
+{
+  ScenePluginModel* scenePluginModel(nullptr);
+  GLWidget* glWidget(nullptr);
+#ifdef AVO_USE_VTK
+  VTK::vtkGLWidget* vtkWidget(nullptr);
+#endif
+  if ((glWidget = qobject_cast<GLWidget*>(m_multiViewWidget->activeWidget()))) {
+    scenePluginModel = &glWidget->sceneModel();
+  }
+#ifdef AVO_USE_VTK
+  else if ((vtkWidget = qobject_cast<VTK::vtkGLWidget*>(
+              m_multiViewWidget->activeWidget()))) {
+    scenePluginModel = &vtkWidget->sceneModel();
+  }
+#endif
+
+  foreach (ScenePlugin* scene, scenePluginModel->scenePlugins())
+    foreach (const QString& name, displayTypes)
+      if (scene->objectName() == name)
+        scene->setEnabled(false);
+
   if (glWidget)
     glWidget->updateScene();
 #ifdef AVO_USE_VTK
@@ -1606,6 +1756,22 @@ void MainWindow::setBackgroundColor()
   }
 }
 
+void MainWindow::setRenderingSettings()
+{
+  Rendering::SolidPipeline* pipeline(nullptr);
+  GLWidget* glWidget(nullptr);
+  if ((glWidget = qobject_cast<GLWidget*>(m_multiViewWidget->activeWidget())))
+    pipeline = &glWidget->renderer().solidPipeline();
+  if (pipeline) {
+    RenderingDialog dialog(this, *pipeline);
+    dialog.exec();
+    QSettings settings;
+    settings.setValue("MainWindow/ao_enabled", pipeline->getAoEnabled());
+    settings.setValue("MainWindow/ao_strength", pipeline->getAoStrength());
+    settings.setValue("MainWindow/ed_enabled", pipeline->getEdEnabled());
+  }
+}
+
 void MainWindow::setProjectionPerspective()
 {
   Rendering::GLRenderer* renderer(nullptr);
@@ -1694,7 +1860,7 @@ void MainWindow::buildMenu()
   QStringList path;
   path << tr("&File");
   // New
-  QAction* action = new QAction(tr("&New"), this);
+  auto* action = new QAction(tr("&New"), this);
   action->setShortcut(QKeySequence::New);
 #ifndef Q_OS_MAC
   action->setIcon(QIcon::fromTheme("document-new"));
@@ -1753,14 +1919,15 @@ void MainWindow::buildMenu()
 #ifndef Q_OS_MAC
   action->setIcon(QIcon::fromTheme("document-export"));
 #endif
-  connect(action, &QAction::triggered, this, &MainWindow::exportFile);
+  connect(action, SIGNAL(triggered()), this, SLOT(exportFile()));
   // Export graphics
   action = new QAction(tr("&Graphics…"), this);
   m_menuBuilder->addAction(exportPath, action, 100);
 #ifndef Q_OS_MAC
   action->setIcon(QIcon::fromTheme("document-export"));
 #endif
-  connect(action, &QAction::triggered, this, &MainWindow::exportGraphics);
+  connect(action, &QAction::triggered, this,
+          static_cast<void (MainWindow::*)()>(&MainWindow::exportGraphics));
 
   // Quit
   action = new QAction(tr("&Quit"), this);
@@ -1814,9 +1981,9 @@ void MainWindow::buildMenu()
   connect(m_undo, &QAction::triggered, this, &MainWindow::undoEdit);
   connect(m_redo, &QAction::triggered, this, &MainWindow::redoEdit);
   connect(m_copyImage, &QAction::triggered, this, &MainWindow::copyGraphics);
-  m_menuBuilder->addAction(editPath, m_undo, 1);
-  m_menuBuilder->addAction(editPath, m_redo, 0);
-  m_menuBuilder->addAction(editPath, m_copyImage, -10);
+  m_menuBuilder->addAction(editPath, m_undo, 999);
+  m_menuBuilder->addAction(editPath, m_redo, 990);
+  m_menuBuilder->addAction(editPath, m_copyImage, 530);
 
   // View menu
   QStringList viewPath;
@@ -1825,6 +1992,10 @@ void MainWindow::buildMenu()
   m_menuBuilder->addAction(viewPath, action, 100);
   connect(action, &QAction::triggered, this, &MainWindow::setBackgroundColor);
 
+  action = new QAction(tr("Rendering…"), this);
+  m_menuBuilder->addAction(viewPath, action, 100);
+  connect(action, &QAction::triggered, this, &MainWindow::setRenderingSettings);
+
   // set default projection
   QSettings settings;
   bool perspective = settings.value("MainWindow/perspective", true).toBool();
@@ -1864,31 +2035,31 @@ void MainWindow::buildMenu()
 
   action = new QAction(tr("&Periodic Table…"), this);
   m_menuBuilder->addAction(extensionsPath, action, 0);
-  QtGui::PeriodicTableView* periodicTable = new QtGui::PeriodicTableView(this);
+  auto* periodicTable = new QtGui::PeriodicTableView(this);
   connect(action, &QAction::triggered, periodicTable, &QWidget::show);
 
   QStringList helpPath;
   helpPath << tr("&Help");
-  QAction* about = new QAction(tr("&About"), this);
+  auto* about = new QAction(tr("&About"), this);
 #ifndef Q_OS_MAC
   about->setIcon(QIcon::fromTheme("help-about"));
 #endif
   m_menuBuilder->addAction(helpPath, about, 500);
   connect(about, &QAction::triggered, this, &MainWindow::showAboutDialog);
 
-  QAction* forum = new QAction(tr("&Discussion Forum"), this);
+  auto* forum = new QAction(tr("&Discussion Forum"), this);
   m_menuBuilder->addAction(helpPath, forum, 200);
   connect(forum, &QAction::triggered, this, &MainWindow::openForum);
 
-  QAction* website = new QAction(tr("&Avogadro Website"), this);
+  auto* website = new QAction(tr("&Avogadro Website"), this);
   m_menuBuilder->addAction(helpPath, website, 40);
   connect(website, &QAction::triggered, this, &MainWindow::openWebsite);
 
-  QAction* bug = new QAction(tr("&Report a Bug"), this);
+  auto* bug = new QAction(tr("&Report a Bug"), this);
   m_menuBuilder->addAction(helpPath, bug, 20);
   connect(bug, &QAction::triggered, this, &MainWindow::openBugReport);
 
-  QAction* feature = new QAction(tr("&Suggest a Feature"), this);
+  auto* feature = new QAction(tr("&Suggest a Feature"), this);
   m_menuBuilder->addAction(helpPath, feature, 10);
   connect(feature, &QAction::triggered, this, &MainWindow::openFeatureRequest);
 
@@ -1939,7 +2110,7 @@ void MainWindow::buildTools()
   QList<ToolPluginFactory*> toolPluginFactories =
     plugin->pluginFactories<ToolPluginFactory>();
   foreach (ToolPluginFactory* factory, toolPluginFactories) {
-    ToolPlugin* tool = factory->createInstance();
+    ToolPlugin* tool = factory->createInstance(QCoreApplication::instance());
     tool->setParent(this);
     if (tool)
       m_tools << tool;
@@ -1961,11 +2132,15 @@ void MainWindow::buildTools()
     m_toolToolBar->addAction(action);
 
     connect(action, &QAction::triggered, this, &MainWindow::toolActivated);
+    connect(toolPlugin, &ToolPlugin::registerCommand, this,
+            &MainWindow::registerToolCommand);
+    toolPlugin->registerCommands();
+
     ++index;
   }
 
   // add the fake tooltip method
-  ToolTipFilter* filter = new ToolTipFilter(this);
+  auto* filter = new ToolTipFilter(this);
   foreach (QToolButton* button, m_toolToolBar->findChildren<QToolButton*>()) {
     button->installEventFilter(filter);
   }
@@ -1997,14 +2172,11 @@ QString MainWindow::generateFilterString(
   QString result;
 
   // Create a map that groups the file extensions by name:
-  QMap<QString, QString> formatMap;
-  for (vector<const FileFormat*>::const_iterator it = formats.begin(),
-                                                 itEnd = formats.end();
-       it != itEnd; ++it) {
-    QString name(QString::fromStdString((*it)->name()));
-    vector<string> exts = (*it)->fileExtensions();
-    for (vector<string>::const_iterator eit = exts.begin(), eitEnd = exts.end();
-         eit != eitEnd; ++eit) {
+  QMultiMap<QString, QString> formatMap;
+  for (auto format : formats) {
+    QString name(QString::fromStdString(format->name()));
+    vector<string> exts = format->fileExtensions();
+    for (auto eit = exts.begin(), eitEnd = exts.end(); eit != eitEnd; ++eit) {
       QString ext(QString::fromStdString(*eit));
       if (!formatMap.values(name).contains(ext)) {
         formatMap.insertMulti(name, ext);
@@ -2094,8 +2266,7 @@ void MainWindow::registerMoleQueue()
 
   // Create patterns list
   QList<QRegExp> patterns;
-  for (StringList::const_iterator it = exts.begin(), itEnd = exts.end();
-       it != itEnd; ++it) {
+  for (auto it = exts.begin(), itEnd = exts.end(); it != itEnd; ++it) {
     patterns << QRegExp(extensionToWildCard(QString::fromStdString(*it)),
                         Qt::CaseInsensitive, QRegExp::Wildcard);
   }
@@ -2115,33 +2286,131 @@ void MainWindow::showAboutDialog()
   about.exec();
 }
 
+void MainWindow::openURL(const QString& url)
+{
+#if defined(Q_OS_MAC) || defined(Q_OS_WIN)
+  QDesktopServices::openUrl(url);
+#else
+  // AppImage can't use QDesktopServices::openUrl, so we use QProcess:
+  QProcess::execute(QString("xdg-open %1").arg(url));
+#endif
+}
+
+void MainWindow::checkUpdate()
+{
+  if (m_network == nullptr) {
+    m_network = new QNetworkAccessManager(this);
+    connect(m_network, SIGNAL(finished(QNetworkReply*)), this,
+            SLOT(finishUpdateRequest(QNetworkReply*)));
+  }
+
+  m_network->get(
+    QNetworkRequest(QUrl("https://api.github.com/repos/openchemistry/"
+                         "avogadrolibs/releases/latest")));
+}
+
+void MainWindow::finishUpdateRequest(QNetworkReply* reply)
+{
+  if (!reply->isReadable()) {
+    MESSAGEBOX::warning(this, tr("Network Download Failed"),
+                        tr("Network timeout or other error."));
+    reply->deleteLater();
+    return;
+  }
+
+  auto replyData = reply->readAll();
+
+  QJsonDocument releaseJson = QJsonDocument::fromJson(replyData);
+  auto releaseObject = releaseJson.object();
+  auto latestRelease = releaseObject["tag_name"].toString();
+
+  QSettings settings;
+  QString lastVersion =
+    settings.value("currentVersion", AvogadroApp_VERSION).toString();
+  // could be something like 1.97.0-36-gcd224f0
+
+  // qDebug() << " update comparing " << lastVersion << " to " << latestRelease;
+  QStringList releaseComponents = latestRelease.split('.');
+  QStringList currentComponents = lastVersion.split('.');
+  if (releaseComponents.size() != 3 || currentComponents.size() != 3)
+    // something is very wrong
+    return;
+
+  if (currentComponents[0] > releaseComponents[0] ||
+      (currentComponents[0] == releaseComponents[0] &&
+       currentComponents[1] > releaseComponents[1]))
+    // no update needed
+    return;
+
+  if (currentComponents[0] == releaseComponents[0] &&
+      currentComponents[1] == releaseComponents[1] &&
+      currentComponents[2] >= releaseComponents[2]) {
+    // this will work for like "0-36-whatever" > "0" but not "1"
+    return;
+  }
+
+  // ask the user about fetching the latest release
+  // download, skip, release notes, cancel
+  // skip = save latestRelease in settings
+  QString currentVersion = tr("Your version: %1").arg(AvogadroApp_VERSION);
+  QString newVersion = tr("New version: %1").arg(latestRelease);
+  QString text =
+    tr("An update is available, do you want to download it now?\n");
+  text += currentVersion + '\n' + newVersion;
+  auto result = MESSAGEBOX::information(this, tr("Version Update"), text,
+                                        QMessageBox::Ok | QMessageBox::Ignore |
+                                          QMessageBox::Cancel);
+
+  if (result == QMessageBox::Cancel)
+    return;
+
+  if (result == QMessageBox::Ignore) {
+    settings.setValue("currentVersion", latestRelease);
+    return;
+  }
+
+  // get an update
+#if defined(Q_OS_MAC)
+  QString url = QString("https://github.com/OpenChemistry/avogadrolibs/"
+                        "releases/download/%1/Avogadro2-%2-Darwin.dmg")
+                  .arg(latestRelease)
+                  .arg(latestRelease);
+#elif defined(Q_OS_WIN)
+  QString url = QString("https://github.com/OpenChemistry/avogadrolibs/"
+                        "releases/download/%1/Avogadro2-%2-win64.exe")
+                  .arg(latestRelease)
+                  .arg(latestRelease);
+#else
+  QString url("https://github.com/OpenChemistry/avogadrolibs/releases/latest");
+#endif
+  openURL(url);
+}
+
 void MainWindow::openForum()
 {
-  QDesktopServices::openUrl(QUrl("https://discuss.avogadro.cc/"));
+  openURL("https://discuss.avogadro.cc/");
 }
 
 void MainWindow::openWebsite()
 {
-  QDesktopServices::openUrl(QUrl("https://two.avogadro.cc/"));
+  openURL("https://two.avogadro.cc/");
 }
 
 void MainWindow::openBugReport()
 {
-  QDesktopServices::openUrl(
-    QUrl("https://github.com/OpenChemistry/avogadrolibs/issues/"
-         "new?template=bug_report.md"));
+  openURL("https://github.com/OpenChemistry/avogadrolibs/issues/"
+          "new?template=bug_report.md");
 }
 
 void MainWindow::openFeatureRequest()
 {
-  QDesktopServices::openUrl(
-    QUrl("https://github.com/OpenChemistry/avogadrolibs/issues/"
-         "new?template=feature_request.md"));
+  openURL("https://github.com/OpenChemistry/avogadrolibs/issues/"
+          "new?template=feature_request.md");
 }
 
 void MainWindow::fileFormatsReady()
 {
-  ExtensionPlugin* extension(qobject_cast<ExtensionPlugin*>(sender()));
+  auto* extension(qobject_cast<ExtensionPlugin*>(sender()));
   if (!extension)
     return;
   foreach (FileFormat* format, extension->fileFormats()) {
@@ -2185,4 +2454,92 @@ void MainWindow::clearQueuedFiles()
   }
 }
 
+void MainWindow::registerToolCommand(QString command, QString description)
+{
+  if (m_toolCommandMap.contains(command))
+    return;
+
+  m_commandDescriptionsMap.insert(command, description);
+
+  // get the calling plugin
+  auto* tool(qobject_cast<ToolPlugin*>(sender()));
+  if (!tool)
+    return;
+
+  m_toolCommandMap.insert(command, tool);
+}
+
+void MainWindow::registerExtensionCommand(QString command, QString description)
+{
+  if (m_extensionCommandMap.contains(command))
+    return;
+
+  m_commandDescriptionsMap.insert(command, description);
+
+  // get the calling plugin
+  auto* extension(qobject_cast<ExtensionPlugin*>(sender()));
+  if (!extension)
+    return;
+
+  m_extensionCommandMap.insert(command, extension);
+}
+
+bool MainWindow::handleCommand(const QString& command,
+                               const QVariantMap& options)
+{
+  // handle a few basic commands
+  if (command == "setProjection") {
+    if (options.contains("type")) {
+      QString type = options.value("type").toString();
+      if (type == "perspective")
+        setProjectionPerspective();
+      else if (type == "orthographic")
+        setProjectionOrthographic();
+    }
+    if (options.contains("perspective"))
+      setProjectionPerspective();
+    else if (options.contains("orthographic"))
+      setProjectionOrthographic();
+    return true;
+  } else if (command == "setRenderTypes") {
+    QStringList enableTypes, disableTypes;
+    if (options.contains("types")) {
+      enableTypes = options.value("types").toStringList();
+    } else {
+      // list of true / false types
+      for (auto key : options.keys()) {
+        if (options.value(key).toBool())
+          enableTypes << key;
+        else
+          disableTypes << key;
+      }
+    }
+    setActiveDisplayTypes(enableTypes);
+    setDisabledDisplayTypes(disableTypes);
+    return true;
+  }
+
+  // pass any remaining commands to the tools or extensions
+  if (m_toolCommandMap.contains(command)) {
+    // get the active widget
+    GLWidget* glWidget =
+      qobject_cast<GLWidget*>(m_multiViewWidget->activeWidget());
+
+    if (glWidget == nullptr)
+      return false;
+
+    auto* tool = m_toolCommandMap.value(command);
+    auto* currentTool = glWidget->activeTool();
+    glWidget->setActiveTool(tool->objectName());
+    bool result = tool->handleCommand(command, options);
+    glWidget->setActiveTool(currentTool);
+
+    return result;
+  } else if (m_extensionCommandMap.contains(command)) {
+    auto* extension = m_extensionCommandMap.value(command);
+    return extension->handleCommand(command, options);
+  }
+  return false;
+}
+
 } // End of Avogadro namespace
