@@ -7,6 +7,7 @@
 #define AVOGADRO_MAINWINDOW_H
 
 #include <QtCore/QStringList>
+#include <QtCore/QVariantMap>
 #include <QtWidgets/QMainWindow>
 
 #ifdef QTTESTING
@@ -16,6 +17,8 @@ class pqTestUtility;
 class QProgressDialog;
 class QThread;
 class QTreeView;
+class QNetworkAccessManager;
+class QNetworkReply;
 
 namespace Ui {
 class AboutDialog;
@@ -31,7 +34,7 @@ namespace QtOpenGL {
 class GLWidget;
 }
 
-namespace Io {
+namespace Io { 
 class FileFormat;
 }
 
@@ -46,6 +49,9 @@ class RWMolecule;
 class LayerModel;
 }
 
+#ifdef _3DCONNEXION
+class TDxController;
+#endif
 /**
  * @class MainWindow
  * @author Marcus D. Hanwell
@@ -85,6 +91,27 @@ public slots:
    */
   bool openFile(const QString& fileName, Io::FileFormat* reader = nullptr);
 
+  void exportGraphics(QString fileName);
+
+  /**
+   * Export a file, using the full selection of formats capable of writing.
+   * The format will be guessed based on the filename extension.
+   * If @a async is true (default), the file is saved asynchronously.
+   * @return If @a async is true, this function returns true if a suitable
+   * writer was found (not if the write was successful). If @a async is
+   * false, the return value indicates whether or not the file was written
+   * successfully.
+   */
+  bool exportFile(const QString& fileName, bool async = true);
+
+  /**
+   * Export a file, using the full selection of formats capable of writing.
+   * Will use @a format to determine the file format to use.
+   * @return String-representation of the exported file, or an empty string if
+   * the export failed.
+   */
+  std::string exportString(const std::string& format);
+
   /**
    * Move @a fileName as a plugin script (i.e. put it in the correct dir)
    */
@@ -118,6 +145,15 @@ public:
     m_localeCodes = codes;
   }
 
+  /**
+   * Handle script commands
+   * @param command The command to execute
+   * @param options The options to the command
+   *
+   * @return True if the command was handled, false otherwise
+   */
+  bool handleCommand(const QString& command, const QVariantMap& options);
+
 signals:
   /**
    * Emitted when the active molecule in the application has changed.
@@ -174,7 +210,7 @@ protected slots:
   /**
    * Save the current molecule to its current fileName. If it is not a standard
    * format, offer to export and warn about possible data loss.
-   * If @a async is true (default), the file is loaded asynchronously.
+   * If @a async is true (default), the file is saved asynchronously.
    * @return If @a async is true, this function returns true if a suitable
    * writer was found (not if the write was successful). If @a async is
    * false, the return value indicates whether or not the file was written
@@ -185,7 +221,7 @@ protected slots:
   /**
    * Prompt for a file location, and attempt to save the active molecule to the
    * specified location.
-   * If @a async is true (default), the file is loaded asynchronously.
+   * If @a async is true (default), the file is saved asynchronously.
    * @return If @a async is true, this function returns true if a suitable
    * writer was found (not if the write was successful). If @a async is
    * false, the return value indicates whether or not the file was written
@@ -195,7 +231,7 @@ protected slots:
 
   /**
    * Export a file, using the full selection of formats capable of writing.
-   * If @a async is true (default), the file is loaded asynchronously.
+   * If @a async is true (default), the file is saved asynchronously.
    * @return If @a async is true, this function returns true if a suitable
    * writer was found (not if the write was successful). If @a async is
    * false, the return value indicates whether or not the file was written
@@ -206,7 +242,7 @@ protected slots:
   /**
    * If specified, use the FileFormat @a writer to save the file. This method
    * takes ownership of @a writer and will delete it before returning.
-   * If @a async is true (default), the file is loaded asynchronously.
+   * If @a async is true (default), the file is saved asynchronously.
    * @return If @a async is true, this function returns true if the write begins
    * successfully (not if the writer completes). If @a async is
    * false, the return value indicates whether or not the file was written
@@ -226,6 +262,7 @@ protected slots:
    * @param displayTypes A list of
    */
   void setActiveDisplayTypes(QStringList displayTypes);
+  void setDisabledDisplayTypes(QStringList displayTypes);
 
   void undoEdit();
   void redoEdit();
@@ -245,6 +282,8 @@ private slots:
 
   void showLanguageDialog();
 
+  void openURL(const QString& url);
+
   void openForum();
 
   void openWebsite();
@@ -253,6 +292,14 @@ private slots:
 
   void openFeatureRequest();
 
+  void checkUpdate();
+
+  void finishUpdateRequest(QNetworkReply*);
+
+  void registerToolCommand(QString command, QString description);
+
+  void registerExtensionCommand(QString command, QString description);
+
   /**
    * @brief Register file formats from extensions when ready.
    */
@@ -337,6 +384,8 @@ private slots:
 
   void setBackgroundColor();
 
+  void setRenderingSettings();
+
   void setProjectionOrthographic();
 
   void setProjectionPerspective();
@@ -378,8 +427,15 @@ private:
   QDockWidget* m_toolDock;
   QDockWidget* m_viewDock;
   QDockWidget* m_sceneDock;
+  QDockWidget* m_layerDock;
+  QDockWidget* m_moleculeDock;
   QList<QtGui::ToolPlugin*> m_tools;
   QList<QtGui::ExtensionPlugin*> m_extensions;
+  // map from script commands to tools and extensions
+  QMap<QString, QtGui::ToolPlugin*> m_toolCommandMap;
+  QMap<QString, QtGui::ExtensionPlugin*> m_extensionCommandMap;
+  // used for help - provide description for a command
+  QMap<QString, QString> m_commandDescriptionsMap;
 
   QAction* m_undo;
   QAction* m_redo;
@@ -389,6 +445,11 @@ private:
 
   ViewFactory* m_viewFactory;
 
+  QNetworkAccessManager* m_network = nullptr;
+#ifdef _3DCONNEXION
+  TDxController* m_TDxController;
+#endif
+
 #ifdef QTTESTING
   pqTestUtility* m_testUtility;
   QString m_testFile;
